# -*- coding: cp1250 -*-

import MySQLdb #needed for cursor
import time
import os
import threading


class data_acquisition_thread(threading.Thread):
    def __init__(self,connection):
        super(data_acquisition_thread, self).__init__()
        self.connection=connection
        self._stop = threading.Event()

        self.path=''   #path to statdata folder
        self.file_nr=0
        
        lock.acquire()
        cursor=self.connection.cursor(MySQLdb.cursors.DictCursor)
        cursor.execute('select * from batches order by live desc, timestamp desc limit 1')
        row=cursor.fetchone()
        lock.release()
        
        if row['live']:
            self.acquire_batch_finished=0
            self.current_batch_id=row['id']
        else:
            self.acquire_batch_finished=1
            self.current_batch_id=None

    def stop(self):
        self._stop.set()

    def stopped(self):
        return self._stop.isSet()
    
    def run(self):
        while not self.stopped():
            self.acquire_all_data()
            
            time.sleep(2)

    def acquire_all_data(self): #acquire all available data
        print 'acquire_all_data'
        
        #new batch began
        if self.acquire_batch_finished and not os.path.exists(self.path+'linempty.dat'): 
            self.file_nr=0
            self.acquire_batch_finished=0

            lock.acquire()
            cursor=self.connection.cursor()
            cursor.execute("insert into batches set timestamp='"+str(int(time.time()))+"', live=1")
            cursor.close()

            self.current_batch_id=self.connection.insert_id()

            self.connection.commit()
            lock.release()
       
        repeat=1
        while repeat and not self.acquire_batch_finished:
            file_name=self.file_nr2name(self.file_nr)
            if os.path.exists(self.path+self.file_nr2name(self.file_nr+1)):
                #file is not last in existing set
                self.acquire_file(self.path+file_name)
                self.file_nr+=1
            elif os.path.exists(self.path+'linempty.dat') and os.path.exists(self.path+file_name):
                #file is last in existing set and line is empty (=no more data
                #                                               will be written)
                self.acquire_batch_finished=1
                self.acquire_file(self.path+file_name)

                lock.acquire()
                cursor=self.connection.cursor()
                cursor.execute("update batches set live=0 where id="+str(self.current_batch_id))
                cursor.close()
                
                self.connection.commit()
                lock.release()

            else:
                repeat=0


    
    def acquire_file(self,file_name):
        f = open(file_name,"r")
        
        if file_name=='00000000.dat':    #read header with meas. limits in first file
            line=f.readline()
            values=line.split(' ',4)
            """ 0 - meas_17_min
                1 - meas_17_max
                2 - meas_18_1_min
                3 - meas_18_1_max
                4 - meas_18_2_min
            """
            #print values
            sql="update batches set "
            sql+="meas_17_min='"+values[0]+"',"
            sql+="meas_17_max='"+values[1]+"',"
            sql+="meas_18_1_min='"+values[2]+"',"
            sql+="meas_18_1_max='"+values[3]+"',"
            sql+="meas_18_2_min='"+values[4]+"' "
            sql+="where id="+str(self.current_batch_id)

            lock.acquire()
            cursor=self.connection.cursor()
            cursor.execute(sql)
            cursor.close()

            self.connection.commit()
            lock.release()
            
            
        for line in f:
            line=line.strip('\r\n')
            values=line.split(' ',3)
            """ to be added - batch nr
                0 - meas_17
                1 - meas_18_1
                2 - meas_18_2
                3 - Y M D h m s
            """
            #print values
            timestamp=int(time.mktime(time.strptime(values[3],"%Y %m %d %H %M %S")))

            sql="insert into measurements set "
            sql+="batch='"+str(self.current_batch_id)+"',"
            sql+="meas_17='"+values[0]+"',"
            sql+="meas_18_1='"+values[1]+"',"
            sql+="meas_18_2='"+values[2]+"',"
            sql+="timestamp='"+str(timestamp)+"'"

            lock.acquire()
            cursor = self.connection.cursor()
            cursor.execute(sql)
            cursor.close()

            self.connection.commit()
            lock.release()

            
        f.close()

    def file_nr2name(self,file_nr):
        file_name=str(file_nr)
        while len(file_name)<8:
            file_name='0'+file_name
        file_name=file_name+'.dat'
        return file_name


def check_connection():
    return os.path.exists(path+'map.dat')
              
