import MySQLdb
import wx
import gui_my
import time
import sys
from datetime import datetime
import threading

#matplotlib
import matplotlib
matplotlib.use('WXAgg')
from matplotlib.figure import Figure
from matplotlib.backends.backend_wxagg import \
    FigureCanvasWxAgg as FigureCanvas, \
    NavigationToolbar2WxAgg as NavigationToolbar
import pylab



class measData(): #data from single measurement device
    def __init__(self,batchId,measName,minValue,maxValue,listColumnIds):
        """ set maxValue to -1 if not existent """

        self.batchId=batchId
        self.measName=measName
        self.values=[]
        self.valuesOK=[]
        self.count=0
        self.countOK=0
        
        self.newValues=[]    #"buffer"
        self.newValuesOK=[]
        self.newCount=0
        self.newCountOK=0
        
        self.minValue=minValue
        self.maxValue=maxValue
        self.listColumnIds=listColumnIds

        spp=matplotlib.figure.SubplotParams(left=0.05,right=1,bottom=0.1,top=0.95)
        self.figure=Figure((8.00,1.50),dpi=100,subplotpars=spp)
        self.figure.set_facecolor('w')


        exec 'self.canvas=FigureCanvas(mainFrame.'+self.measName+'_GraphPanel,-1,self.figure)'

        self.axes=self.figure.add_subplot(1,1,1,)
        self.axes.set_ymargin(0.1)

        pylab.setp(self.axes.get_xticklabels(), fontsize=7)
        pylab.setp(self.axes.get_yticklabels(), fontsize=7)
        

    def append(self,parent,index,value):
        """
        adds a value to the measures
        returns true if value in limits, false if not
        """
        
        self.newValues.append(value)
        self.newCount+=1

        mainFrame.dataListCtrl.SetStringItem(index,parent.listColumnIds[self.measName],str(value))

        mainFrame.dataListCtrl.SetStringItem(index,parent.listColumnIds[self.measName+'_min'],str(self.minValue))
        if self.maxValue!=-1:
            mainFrame.dataListCtrl.SetStringItem(index,parent.listColumnIds[self.measName+'_max'],str(self.maxValue))
            
        if value>self.minValue and (value<self.maxValue or self.maxValue==-1):
            self.newValuesOK.append(value)
            self.newCountOK+=1
            measOK=1
            status='OK'
        else:
            measOK=0
            status='NOK'
            
        mainFrame.dataListCtrl.SetStringItem(index,parent.listColumnIds[self.measName+'_OK'],status)
        return measOK


    def refreshOverview(self,parent):
        countOK=countNOK=percentNOK=OKAverage=OKMin=OKMax=0
        if parent.measuresCount:
            lock.acquire()
            cursor=connection.cursor(MySQLdb.cursors.DictCursor)
            sql='select * from batchesOverview where batch='+str(self.batchId)+' and measName=\''+self.measName+'\''
            cursor.execute(sql)

            if parent.live or cursor.rowcount==0:
                if self.newCountOK and self.countOK:
                    self.OKAverage=(self.OKAverage*self.countOK+sum(self.newValuesOK))/(self.countOK+self.newCountOK)
                    self.OKMin=min([self.OKMin]+self.newValuesOK)
                    self.OKMax=max([self.OKMax]+self.newValuesOK)
                elif self.newCountOK:
                    self.OKAverage=sum(self.newValues)/self.newCountOK
                    self.OKMin=min(self.newValuesOK)
                    self.OKMax=max(self.newValuesOK)
                else:
                    self.OKAverage=-1
                    self.OKMin=-1
                    self.OKMax=-1

            else:
                result=cursor.fetchone()

                self.OKAverage=result['OKAverage']
                self.OKMin=result['OKMin']
                self.OKMax=result['OKMax']

            self.values+=self.newValues
            self.count+=self.newCount
            self.countOK+=self.newCountOK

            countNOK=self.count-self.countOK
            percentNOK=float(countNOK)/self.count

            self.newValues=[] #"buffer" reset
            self.newValuesOK=[]
            self.newCount=0
            self.newCountOK=0

            if cursor.rowcount==0 and not parent.live:
                sql='insert into batchesOverview set '
                sql+='batch='+str(self.batchId)+', '
                sql+='measName=\''+self.measName+'\', '

                sql+='countOK='+str(self.countOK)+', '
                sql+='countNOK='+str(countNOK)+', '
                sql+='percentNOK='+str(percentNOK)+', '
                sql+='OKAverage='+str(self.OKAverage)+', '
                sql+='OKMin='+str(self.OKMin)+', '
                sql+='OKMax='+str(self.OKMax)+' '
                
                cursor.execute(sql)

            cursor.close()
            lock.release()
                

            self.plot_data = self.axes.plot(range(1,self.count+1),self.values,linewidth=2,color='b')[0]
            self.plot_data = self.axes.plot([1,self.count],[self.minValue,self.minValue],linewidth=1,color='r')[0]
            if self.maxValue!=-1:
                self.plot_data = self.axes.plot([1,self.count],[self.maxValue,self.maxValue],linewidth=1,color='r')[0]
            self.axes.set_xbound(lower=1,upper=self.count+1)
            self.canvas.draw()
            
        exec 'mainFrame.'+self.measName+'_PiecesOKValueText.SetLabel(str(self.countOK))'
        exec 'mainFrame.'+self.measName+'_PiecesNOKValueText.SetLabel(str(countNOK))'
        exec 'mainFrame.'+self.measName+'_PercentNOKValueText.SetLabel(\'%.2f\'%(percentNOK*100)+\'%\')'
        exec 'mainFrame.'+self.measName+'_OKAverageValueText.SetLabel(\'%.2f\'%self.OKAverage)'
        exec 'mainFrame.'+self.measName+'_OKMinValueText.SetLabel(str(self.OKMin))'
        exec 'mainFrame.'+self.measName+'_OKMaxValueText.SetLabel(str(self.OKMax))'


class batchData():
    def __init__(self,batchId):
        #time.sleep(2)
        self.measNames=['meas_17','meas_18_1','meas_18_2']

        #dataList pre-init
        listColumns=['Nr',
                     'Timestamp',
                     'meas_17',
                     'meas_17_min',
                     'meas_17_max',
                     'meas_17_OK',
                     'meas_18_1',
                     'meas_18_1_min',
                     'meas_18_1_max',
                     'meas_18_1_OK',
                     'meas_18_2',
                     'meas_18_2_min',
                     'meas_18_2_OK',
                     'OverallOK']
        i=0
        self.listColumnIds={}
        for col in listColumns:
            self.listColumnIds[col]=i
            i+=1

        #dataList init
        mainFrame.dataListCtrl.InsertColumn(self.listColumnIds['Nr'],'#')
        mainFrame.dataListCtrl.SetColumnWidth(self.listColumnIds['Nr'], 55)
        mainFrame.dataListCtrl.InsertColumn(self.listColumnIds['Timestamp'],'Timestamp')
        mainFrame.dataListCtrl.SetColumnWidth(self.listColumnIds['Timestamp'], 115)
        
        mainFrame.dataListCtrl.InsertColumn(self.listColumnIds['meas_17'],'BW')
        mainFrame.dataListCtrl.SetColumnWidth(self.listColumnIds['meas_17'], 50)
        mainFrame.dataListCtrl.InsertColumn(self.listColumnIds['meas_17_min'],'BW min')
        mainFrame.dataListCtrl.SetColumnWidth(self.listColumnIds['meas_17_min'], 50)
        mainFrame.dataListCtrl.InsertColumn(self.listColumnIds['meas_17_max'],'BW max')
        mainFrame.dataListCtrl.SetColumnWidth(self.listColumnIds['meas_17_max'], 52)
        mainFrame.dataListCtrl.InsertColumn(self.listColumnIds['meas_17_OK'],'')
        mainFrame.dataListCtrl.SetColumnWidth(self.listColumnIds['meas_17_OK'], 33)
        
        mainFrame.dataListCtrl.InsertColumn(self.listColumnIds['meas_18_1'],'SC')
        mainFrame.dataListCtrl.SetColumnWidth(self.listColumnIds['meas_18_1'], 50)
        mainFrame.dataListCtrl.InsertColumn(self.listColumnIds['meas_18_1_min'],'SC min')
        mainFrame.dataListCtrl.SetColumnWidth(self.listColumnIds['meas_18_1_min'], 50)
        mainFrame.dataListCtrl.InsertColumn(self.listColumnIds['meas_18_1_max'],'SC max')
        mainFrame.dataListCtrl.SetColumnWidth(self.listColumnIds['meas_18_1_max'], 50)
        mainFrame.dataListCtrl.InsertColumn(self.listColumnIds['meas_18_1_OK'],'')
        mainFrame.dataListCtrl.SetColumnWidth(self.listColumnIds['meas_18_1_OK'], 33)
        
        mainFrame.dataListCtrl.InsertColumn(self.listColumnIds['meas_18_2'],'Ins')
        mainFrame.dataListCtrl.SetColumnWidth(self.listColumnIds['meas_18_2'], 50)
        mainFrame.dataListCtrl.InsertColumn(self.listColumnIds['meas_18_2_min'],'Ins min')
        mainFrame.dataListCtrl.SetColumnWidth(self.listColumnIds['meas_18_2_min'], 50)
        mainFrame.dataListCtrl.InsertColumn(self.listColumnIds['meas_18_2_OK'],'')
        mainFrame.dataListCtrl.SetColumnWidth(self.listColumnIds['meas_18_2_OK'], 33)
        
        mainFrame.dataListCtrl.InsertColumn(self.listColumnIds['OverallOK'],'Part OK/NOK')
        mainFrame.dataListCtrl.SetColumnWidth(self.listColumnIds['OverallOK'], 75)

        self.resetData(batchId)

    def changeBatch(self,batchId):
        if self.live:
            self.thread.stop()
            
        self.resetData(batchId)

    def resetData(self,batchId):
        """resets everything to default (clear) state, so that all data can be (re)loaded from db"""
        mainFrame.dataListCtrl.DeleteAllItems()

        self.batchId=batchId

        lock.acquire()
        cursor=connection.cursor(MySQLdb.cursors.DictCursor)
        cursor.execute('select * from batches where id='+str(self.batchId))
        result=cursor.fetchone()
        cursor.close()
        lock.release()
        
        self.batchName=result['name']
        self.timestamp=result['timestamp']
        self.live=result['live']

        timestamp=datetime.fromtimestamp(result['timestamp'])
        mainFrame.batchName.SetLabel(self.batchName+timestamp.strftime('%Y-%m-%d %H:%M:%S'))
        
        for measName in self.measNames:
            if measName=='meas_18_2':       #with insulation resistance, there's only minimal value (no maximal)
                exec "self."+measName+"=measData(self.batchId,measName,result['"+measName+"_min'],-1,self.listColumnIds)"
            else:
                exec "self."+measName+"=measData(self.batchId,measName,result['"+measName+"_min'],result['"+measName+"_max'],self.listColumnIds)"

        self.measuresCount=0
        self.maxTimestamp=0

        if self.live:
            self.thread=dataRefreshThread(self)
            self.thread.start()
        else:
            self.refreshData()
        
        
    
    def refreshData(self):
        """loads data from db"""
        print 'loading data'
        lock.acquire()
        cursor=connection.cursor(MySQLdb.cursors.DictCursor)
        cursor.execute('select * from batches where id='+str(self.batchId))
        result=cursor.fetchone()
        cursor.close()
        lock.release()
        if self.live:
            if not result['live']:
                self.thread.stop()
        self.live=result['live']

        lock.acquire()
        cursor=connection.cursor(MySQLdb.cursors.DictCursor)
        cursor.execute('select * from measurements where batch='+str(self.batchId)+' and timestamp>'+str(self.maxTimestamp)+' order by timestamp')
        resultset=cursor.fetchall()
        cursor.close()
        lock.release()
        print '  MySQL fetch finished'

        for row in resultset:
            index=mainFrame.dataListCtrl.InsertStringItem(self.measuresCount,'')
            mainFrame.dataListCtrl.SetStringItem(index,self.listColumnIds['Nr'],str(self.measuresCount+1))
            timestamp=datetime.fromtimestamp(row['timestamp'])
            mainFrame.dataListCtrl.SetStringItem(index,self.listColumnIds['Timestamp'],timestamp.strftime('%Y-%m-%d %H:%M:%S'))


            if (row['meas_17']>0):
                bwOK=self.meas_17.append(self,index,row['meas_17'])
            else:  #meas<=0 - didn't finish measuring or didn't measure at all
                bwOK=0
                
            if (row['meas_18_1']>0):
                scOK=self.meas_18_1.append(self,index,row['meas_18_1'])
            else:  #meas<=0 - didn't finish measuring or didn't measure at all
                scOK=0
                
            if (row['meas_18_2']>0):
                insOK=self.meas_18_2.append(self,index,row['meas_18_2'])
            else:  #meas<=0 - didn't finish measuring or didn't measure at all
                insOK=0
            
            if bwOK and scOK and insOK:
                status='OK'
            else:
                status='NOK'
                mainFrame.dataListCtrl.SetItemBackgroundColour(index,wx.Colour(255,160,160))

            """if not scOK: ## pokus o nastaveni barvy bunky se spatnou hodnotou
                #mainFrame.dataListCtrl.SetItemBackgroundColour(index,wx.Colour(0,0,250))

                item=mainFrame.dataListCtrl.GetItem(index,3)
                item.SetBackgroundColour(wx.BLUE)
                #mainFrame.dataListCtrl.SetItem(item)
                #mainFrame.dataListCtrl.Update()
            """
                
            mainFrame.dataListCtrl.SetStringItem(index,self.listColumnIds['OverallOK'],status)

            if row['timestamp']>self.maxTimestamp:
                self.maxTimestamp=row['timestamp']
            self.measuresCount+=1
        print 'data loaded'

        self.meas_17.refreshOverview(self)
        self.meas_18_1.refreshOverview(self)
        self.meas_18_2.refreshOverview(self)

class dataRefreshThread(threading.Thread):
    def __init__(self,batchData):
        super(dataRefreshThread, self).__init__()

        self.batchData=batchData
        self._stop = threading.Event()

    def stop(self):
        self._stop.set()

    def stopped(self):
        return self._stop.isSet()

    
    def run(self):
        while not self.stopped():
            self.batchData.refreshData()
            time.sleep(10)


def batchSelect(event):
    batchFrame = gui_my.BatchSelectFrame(None,batch)
    batchFrame.Show(True)
    
    lock.acquire()
    cursor=connection.cursor(MySQLdb.cursors.DictCursor)
    cursor.execute('select * from batches order by timestamp desc')
    resultset=cursor.fetchall()
    cursor.close()
    lock.release()

    i=0
    for row in resultset:
        index=batchFrame.batchSelectListCtrl.InsertStringItem(i,'')
        if i==0:
            batchFrame.batchSelectListCtrl.SetItemState(index, wx.LIST_STATE_SELECTED, wx.LIST_STATE_SELECTED)
        
        batchFrame.batchSelectListCtrl.SetStringItem(index,batchFrame.listColumnIds['batch_name'],row['name'])
        timestamp=datetime.fromtimestamp(row['timestamp'])
        batchFrame.batchSelectListCtrl.SetStringItem(index,batchFrame.listColumnIds['timestamp'],timestamp.strftime('%Y-%m-%d %H:%M:%S'))

        if not row['live']:
            for measName in batch.measNames:
                lock.acquire()
                cursor=connection.cursor(MySQLdb.cursors.DictCursor)
                cursor.execute('select * from batchesOverview where batch='+str(row['id'])+' and measName=\''+measName+'\'')
                result=cursor.fetchone()
                cursor.close()
                lock.release()

                for valueName in ['countOK','countNOK','percentNOK','OKAverage','OKMin','OKMax']:
                    batchFrame.batchSelectListCtrl.SetStringItem(index,batchFrame.listColumnIds[measName+'_'+valueName],str(result[valueName]))

        if row['live']:
            batchFrame.batchSelectListCtrl.SetStringItem(index,batchFrame.listColumnIds['live'],'LIVE')

        batchFrame.IdList.append(row['id'])
        
        i+=1

def closeWindow(event):
    if hasattr(batch, 'thread'):
        batch.thread.stop()
    mainFrame.Destroy()


app = wx.App(0)
mainFrame = gui_my.MainFrame(None)
mainFrame.selectBatchButton.Bind(wx.EVT_BUTTON, batchSelect)
mainFrame.Bind(wx.EVT_CLOSE, closeWindow)

