unit statisti;

interface

uses DOS,
     Temporis,
     FileUtil, 
     OBJECTS, {for at least TBufStream}
     IMPDISK,
     DECGLOBA, {for at least ListeAttenteEvenement}
     PROCGLOB;  {for at least ListeAttenteEvenement}

const
  folder='statdata'; 
  
type
  PartMeasures = record
    Measure_17:longint;     {bridge wire resistance}
    Measure_18_1:longint;   {shortcut resistance} 
    Measure_18_2:longint;   {insulation resistance}

    Year:integer;           {timestamp when measures were "buffered" (station 20)}
    Month:integer;          
    Day:integer;
    Hour:integer;
    Minute:integer;
    Second:integer;                
  end;  

var NoMeasWritten:integer;
    NoMeasInBuffer:integer;
    MeasuresBuffer:array[0..256] of PartMeasures;
    
    Measure_17_Min:longint;  {limits}
    Measure_17_Max:longint;
    Measure_18_1_Min:longint;
    Measure_18_1_Max:longint;
    Measure_18_2_Min:longint;
    
    Flag_RunNewBatch:boolean;
    Flag_RunLineEmpty:boolean;

procedure Stat_SetLimits(meas_17_min,meas_17_max,meas_18_1_min,meas_18_1_max,meas_18_2_min:longint);
      {loads limit values into variables}
procedure Stat_CollectData(meas_17,meas_18_1,meas_18_2:longint); 
      {testing}
procedure Stat_WriteData;                                        
      {writes contents of MeasureBuffer to disk and empties MeasureBuffer}
procedure Stat_NewBatch;                                         
      {sets the flag to delete linempty.dat and numbered data files (00000000.dat...)}
procedure Stat_LineEmpty;                                        
      {sets the flag to create linempty.dat}
function Stat_GetNoMeasWritten:integer;
function Stat_GetNoMeasInBuffer:integer;
      
implementation

procedure Stat_SetLimits(meas_17_min,meas_17_max,meas_18_1_min,meas_18_1_max,meas_18_2_min:longint);
  begin
    Measure_17_Min:=meas_17_min;       {load limits}  
    Measure_17_Max:=meas_17_max;  
    Measure_18_1_Min:=meas_18_1_min;  
    Measure_18_1_Max:=meas_18_1_max;  
    Measure_18_2_Min:=meas_18_2_min;
  end;

procedure Stat_WriteData;
  var filename,dummy:string;
      i,j,NoZeros:integer;
      f:text;

  begin
    if (NoMeasInBuffer<>0) then begin

      str(int(NoMeasWritten/10):0:0,filename);
      NoZeros:=8-length(filename);
  
      for j:=1 to NoZeros do begin     {add numbers (for better file }
        filename:='0'+filename;        {             sorting in NC..)}
      end;
            
      filename:=folder+'/'+filename+'.dat';

      assign(f,filename);

      {$I-}
      append(f);	       
      if IOResult <> 0 then begin
        rewrite(f);	       
      end;
      {$I+}
      

      if NoMeasWritten=0 then begin {if this is the first part in the batch, 
                                     write batch header with measure limits}
        {TO BE ADDED - write batch name}
        str(Measure_17_Min:0,dummy);
        write(f,dummy);

        write(f,' ');
        str(Measure_17_Max:0,dummy);
        write(f,dummy);

        write(f,' ');
        str(Measure_18_1_Min:0,dummy);
        write(f,dummy);

        write(f,' ');
        str(Measure_18_1_Max:0,dummy);
        write(f,dummy);

        write(f,' ');
        str(Measure_18_2_Min:0,dummy);
        write(f,dummy);

        writeln(f,'');
      end;
      

      for i:=0 to (NoMeasInBuffer-1) do begin
        str(MeasuresBuffer[i].Measure_17:0,dummy);
        write(f,dummy);

        write(f,' ');
        str(MeasuresBuffer[i].Measure_18_1:0,dummy);
        write(f,dummy);

        write(f,' ');
        str(MeasuresBuffer[i].Measure_18_2:0,dummy);
        write(f,dummy);
        
        write(f,' ');
        str(MeasuresBuffer[i].Year:0,dummy);
        write(f,dummy);

        write(f,' ');
        str(MeasuresBuffer[i].Month:0,dummy);
        write(f,dummy);

        write(f,' ');
        str(MeasuresBuffer[i].Day:0,dummy);
        write(f,dummy);
        
        write(f,' ');
        str(MeasuresBuffer[i].Hour:0,dummy);
        write(f,dummy);

        write(f,' ');
        str(MeasuresBuffer[i].Minute:0,dummy);
        write(f,dummy);

        write(f,' ');
        str(MeasuresBuffer[i].Second:0,dummy);
        write(f,dummy);

        writeln(f,'');
      end;
      close(f);      
      
      NoMeasInBuffer:=0;
      NoMeasWritten:=NoMeasWritten+1;
    end;
  end;

procedure Stat_CollectData(meas_17,meas_18_1,meas_18_2:longint); 
  begin                                
    MeasuresBuffer[NoMeasInBuffer].Measure_17:=meas_17;
    MeasuresBuffer[NoMeasInBuffer].Measure_18_1:=meas_18_1;
    MeasuresBuffer[NoMeasInBuffer].Measure_18_2:=meas_18_2;

    MeasuresBuffer[NoMeasInBuffer].Year:=Annee;
    MeasuresBuffer[NoMeasInBuffer].Month:=Mois;
    MeasuresBuffer[NoMeasInBuffer].Day:=Jour;
    MeasuresBuffer[NoMeasInBuffer].Hour:=Heures;
    MeasuresBuffer[NoMeasInBuffer].Minute:=Minutes;
    MeasuresBuffer[NoMeasInBuffer].Second:=Secondes;
    
    NoMeasInBuffer:=NoMeasInBuffer+1;    
  end;

procedure Stat_NewBatch;
  begin
    Flag_RunNewBatch:=true;
  end;

procedure Stat_RunNewBatch;
  {Searches for data files beginning (numbering) from 00000000.dat upwards and deletes them.
   If file not found, stops searching.
   Deletes linempty.dat.
   Resets NoMeasWritten to 0.}
  var filename :string;
      i,j,NoZeros:integer;
      fexist   :boolean;

  begin
    if Flag_RunNewBatch=true then begin
      Flag_RunNewBatch:=false;
      j:=0;

      fexist:=true;
      repeat
      	str(j:0,filename);
      	NoZeros:=8-length(filename);
      
      	for i:=1 to NoZeros do begin	 {add numbers (for better file }
      	  filename:='0'+filename;	 {	       sorting in NC..)}
      	end;
      
      	filename:=folder+'/'+filename+'.dat';
      	if FileExists(filename) then begin
      	  Remove(filename);
      	end else begin                 
      	  fexist:=false;
      	end;
      
      	j:=j+1;
      until fexist=false;

      Remove(folder+'/linempty.dat');
      NoMeasWritten:=0;
    end;
  end;

procedure Stat_LineEmpty;
  begin
    Flag_RunLineEmpty:=true;
  end;

procedure Stat_RunLineEmpty;
  {Creates file linempty.dat.
   To be called when the line has been emptied.
   The existence of 'linempty.dat' tells the statistics
   evaluation program that it can take all remaining data.}

  var f:text;
  begin
    if Flag_RunLineEmpty=true then begin
      Flag_RunLineEmpty:=false;
      assign(f,folder+'/linempty.dat');
      rewrite(f);
      close(f);
    end;
  end;

function Stat_GetNoMeasWritten:integer;
  begin
    Stat_GetNoMeasWritten:=NoMeasWritten;
  end;

function Stat_GetNoMeasInBuffer:integer;
  begin
    Stat_GetNoMeasInBuffer:=NoMeasInBuffer;
  end;

begin
  Flag_RunNewBatch:=false;
  Flag_RunLineEmpty:=false;
  NoMeasInBuffer:=0;
  NoMeasWritten:=0;

  ListeAttenteEvenement.Insert(@Stat_WriteData);
  ListeAttenteEvenement.Insert(@Stat_RunNewBatch);
  ListeAttenteEvenement.Insert(@Stat_RunLineEmpty);
end.
